from .bin_image import bin_image
from .draw_universal_semicircle import draw_universal_semicircle
from .estimate_and_shift_irf import estimate_and_shift_irf
from .ideal_sample_phasor import ideal_sample_phasor
from .lifetime_image_to_rectangular_points import lifetime_image_to_rectangular_points
from .phasor_calibration import phasor_calibration
from .phasor_to_rectangular_lifetimes_array import phasor_to_rectangular_lifetimes_array
from .phasor_to_rectangular_point import phasor_to_rectangular_point
from .rectangular_to_phasor_lifetimes_array import rectangular_to_phasor_lifetimes_array
from .regionprops_omi import regionprops_omi
from .td_to_fd import td_to_fd

__all__ = [
    "bin_image",
    "draw_universal_semicircle",
    "estimate_and_shift_irf",
    "ideal_sample_phasor",
    "lifetime_image_to_rectangular_points",
    "phasor_calibration",
    "phasor_to_rectangular_lifetimes_array",
    "phasor_to_rectangular_point",
    "rectangular_to_phasor_lifetimes_array",
    "td_to_fd",
    "regionprops_omi",
]



# Dependencies
import collections as coll

import matplotlib.pylab as plt
import numpy as np
import pandas as pd
import pylab
import tifffile
from scipy.signal import convolve

from cell_analysis_tools.flim import ideal_sample_phasor, td_to_fd
from cell_analysis_tools.image_processing import normalize
from cell_analysis_tools.io import read_asc


def phasor_calibration(f, lifetime, timebins, counts):
    """ Phasor Plot Calibration

    Parameters
    ---------- 
        f  : int 
            laser repetition angular frequency
        lifetime  : int 
            lifetime of known sample in ns (single exponential decay)
        timebins  : int 
            timebins of samples
        counts  : int 
            photon counts of histogram
    Returns
    -------
        angle_offset {float}: difference in angle between known sample and actual
        magnitude_offset {float}: difference in magnitude between known sample and actual
    Note
    ----
        If no timebins or histograms passed then returns angle and phase of
        lifetime passed in
    """
    Calibration = coll.namedtuple("Calibration", "angle scaling_factor")

    # calculate idea and real phasors
    ideal_sampl_phasor = ideal_sample_phasor(f, lifetime)
    real = td_to_fd(f, timebins, counts)

    """ calculate angle offset """
    angle = ideal_sampl_phasor.angle - real.angle

    """ ratio of magnitudes -> ideal/actual """
    ratio = ideal_sampl_phasor.magnitude / real.magnitude

    return Calibration(angle=angle, scaling_factor=ratio)



import scipy.ndimage as ndi


def fill_and_label_rois(curr_nuclei):
    """
    Fills and labels ROI outlines using unique ints for each region.

    Parameters
    ---------- 
    param curr_nucelei : ndarray
        Current image to process

    Returns
    -------
    output : ndarray 
        ROIs filled and labeled with unique int representations

    """

    return ndi.label(ndi.binary_fill_holes(curr_nuclei))[0]



import collections as coll

import matplotlib.pylab as plt
import numpy as np
import pandas as pd
import pylab
import tifffile
from scipy.signal import convolve

from cell_analysis_tools.image_processing import normalize
from cell_analysis_tools.io import read_asc


def ideal_sample_phasor(f, lifetime):
    """
    Generates a phasor for a specific lifetime at a given frequency.

    Parameters
    ----------
        f : int
            laser rep rate
        lifetime : float
            lifetime of desired single exponential sample

    Returns
    -------
        angle : float
            angle of ideal sample
        magnitude : float
            magnitude of ideal sample
    """

    Phasor = coll.namedtuple("Phasor", "angle magnitude")

    # lifetime = lifetime * 1e-12  # lifetime in ns
    w = 2 * np.pi * f
    ### simulated point values
    ideal_g = 1 / (1 + (w ** 2 * lifetime ** 2))
    ideal_s = (w * lifetime) / (1 + (w * lifetime) ** 2)

    # angle = np.arctan(ideal_s/ideal_g) # radians
    angle = np.pi - np.arctan2(ideal_s, -ideal_g)  # in radians
    magnitude = np.sqrt(ideal_g ** 2 + ideal_s ** 2)

    print("Input lifetime: ", (1 / w * ideal_s / ideal_g))
    return Phasor(angle=angle, magnitude=magnitude)
